package org.pwte.example.ibatis;

import java.sql.SQLException;

import org.pwte.example.domain.Order;

import com.ibatis.sqlmap.client.extensions.ParameterSetter;
import com.ibatis.sqlmap.client.extensions.ResultGetter;
import com.ibatis.sqlmap.client.extensions.TypeHandlerCallback;

public class StatusEnumTypeHandler implements TypeHandlerCallback {

	private static final String 
		OPEN = "OPEN",
		SUBMITTED = "SUBMITTED", 
		CLOSED = "CLOSED"; 

	public Object getResult(ResultGetter getter) throws SQLException {
		String status = getter.getString();
		
		if(status == null)
		{
			return null;
		}
		else if(OPEN.equals(status)) {
			return Order.Status.OPEN;
		} else if(SUBMITTED.equals(status)) {
			return Order.Status.SUBMITTED;
		} else if(CLOSED.equals(status)) {
			return Order.Status.CLOSED;
		} else {
			throw new SQLException("ORDER STATUS fetched from database is: "
					+status
					+" \n STATUS must be OPEN, SUBMITTED, or CLOSED");
		}
	}

	public void setParameter(ParameterSetter setter, Object parameter)
			throws SQLException {
		Order.Status status = (Order.Status) parameter;
		if(status.equals(Order.Status.OPEN)) {
			setter.setString(OPEN);
		} else if(status.equals(Order.Status.SUBMITTED)) {
			setter.setString(SUBMITTED);
		} else {
			setter.setString(CLOSED);
		}
	}

	public Object valueOf(String s) {
		if(OPEN.equals(s)) {
			return Order.Status.OPEN;
		} else if(SUBMITTED.equals(s)) {
			return Order.Status.SUBMITTED;
		} else {
			return Order.Status.CLOSED;
		}
	}

}
