package org.pwte.example.service;

import org.pwte.example.domain.AbstractCustomer;
import org.pwte.example.domain.LineItem;
import org.pwte.example.domain.Order;
import org.pwte.example.exception.CustomerDoesNotExistException;
import org.pwte.example.exception.GeneralPersistenceException;
import org.pwte.example.exception.InvalidQuantityException;
import org.pwte.example.exception.NoLineItemsException;
import org.pwte.example.exception.OrderAlreadyOpenException;
import org.pwte.example.exception.OrderNotOpenException;
import org.pwte.example.exception.ProductDoesNotExistException;

public interface CustomerOrderServices {
	
	/**
	 * @param customerId
	 * @return
	 * @throws CustomerDoesNotExistException
	 * @throws GeneralPersistenceException
	 */
	public AbstractCustomer loadCustomer(int customerId)throws CustomerDoesNotExistException, GeneralPersistenceException;

	/**
	 * @param customerId
	 * @return
	 * @throws CustomerDoesNotExistException
	 * @throws OrderAlreadyOpenException
	 * @throws GeneralPersistenceException
	 */
	public Order openOrder(int customerId) throws CustomerDoesNotExistException, OrderAlreadyOpenException, GeneralPersistenceException; 

	/**
	 * @param customerId
	 * @param productId
	 * @param quantity
	 * @return
	 * @throws CustomerDoesNotExistException
	 * @throws OrderNotOpenException
	 * @throws ProductDoesNotExistException
	 * @throws GeneralPersistenceException
	 */
	public LineItem addLineItem(int customerId, int productId, long quantity) throws CustomerDoesNotExistException, OrderNotOpenException, ProductDoesNotExistException, InvalidQuantityException,GeneralPersistenceException; 

	/**
	 * @param customerId
	 * @throws CustomerDoesNotExistException
	 * @throws OrderNotOpenException
	 * @throws NoLineItemsException
	 * @throws GeneralPersistenceException
	 */
	public void submit(int customerId) throws CustomerDoesNotExistException, OrderNotOpenException, NoLineItemsException, GeneralPersistenceException;

	/**
	 * @param customerId
	 * @param productId
	 * @throws ProductDoesNotExistException
	 * @throws CustomerDoesNotExistException
	 * @throws OrderNotOpenException
	 * @throws InvalidQuantityException
	 * @throws NoLineItemsException
	 * @throws GeneralPersistenceException
	 */
	public void removeLineItem(int customerId, int productId) throws ProductDoesNotExistException, CustomerDoesNotExistException,OrderNotOpenException, InvalidQuantityException, NoLineItemsException, GeneralPersistenceException;
}