package org.pwte.example.domain;

import static javax.persistence.DiscriminatorType.STRING;
import static javax.persistence.InheritanceType.SINGLE_TABLE;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Entity;
import javax.persistence.EntityResult;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.JoinColumn;
import javax.persistence.JoinColumns;
import javax.persistence.NamedNativeQuery;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SqlResultSetMapping;
import javax.persistence.SqlResultSetMappings;
import javax.persistence.Table;

import org.apache.openjpa.persistence.FetchAttribute;
import org.apache.openjpa.persistence.FetchGroup;
import org.apache.openjpa.persistence.FetchGroups;

@Entity
@Inheritance(strategy=SINGLE_TABLE)
@Table(name = "CUSTOMER")
@DiscriminatorColumn(name="TYPE", discriminatorType = STRING)
public abstract class AbstractCustomer implements Serializable {
	
	@Id
	@Column(name="CUSTOMER_ID")
	protected int customerId;
	protected String name;
	protected String type;
	
	@OneToOne(fetch=FetchType.EAGER,cascade = {CascadeType.MERGE,CascadeType.REFRESH},optional=true )
	@JoinColumn(name="OPEN_ORDER", referencedColumnName = "ORDER_ID")
	protected Order openOrder;
	
	@OneToMany(mappedBy="customer",fetch=FetchType.LAZY)
	protected Set<Order> orders;

	public AbstractCustomer(){}
	
	public AbstractCustomer(String name, Order openOrder, Set<Order> orders) {
		super();
		this.name = name;
		this.openOrder = openOrder;
		this.orders = orders;
	}
	
	public int getCustomerId() {return customerId;}
	public void setCustomerId(int customerId) {this.customerId = customerId;}
	public String getName() {return name;}
	public void setName(String name) {this.name = name;}
	public Order getOpenOrder() {return openOrder;}
	public void setOpenOrder(Order openOrder) {this.openOrder = openOrder;}
	public Set<Order> getOrders() {return orders;}
	public void setOrders(Set<Order> orders) {this.orders = orders;}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

}
