package org.pwte.example.service;

import javax.persistence.EntityManager;
import javax.persistence.Persistence;

import org.pwte.example.domain.AbstractCustomer;
import org.pwte.example.domain.LineItem;
import org.pwte.example.domain.Order;
import org.pwte.example.exception.CustomerDoesNotExistException;
import org.pwte.example.exception.GeneralPersistenceException;
import org.pwte.example.exception.InvalidQuantityException;
import org.pwte.example.exception.NoLineItemsException;
import org.pwte.example.exception.OrderAlreadyOpenException;
import org.pwte.example.exception.OrderNotOpenException;
import org.pwte.example.exception.ProductDoesNotExistException;


public class CustomerOrderServicesJavaSEImpl extends CustomerOrderServicesImpl {

	
	
	public CustomerOrderServicesJavaSEImpl() {
		super();
		em = Persistence.createEntityManagerFactory("pie-db-JAVA-SE").createEntityManager();
	}

	public LineItem addLineItem(int customerId, int productId, long quantity)
			throws CustomerDoesNotExistException, OrderNotOpenException,
			ProductDoesNotExistException, GeneralPersistenceException, InvalidQuantityException {
		
		try
		{
			em.getTransaction().begin();
			LineItem lineItem = super.addLineItem(customerId, productId, quantity);
			em.getTransaction().commit();	
			return lineItem;
		}
		catch(CustomerDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(OrderNotOpenException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(ProductDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(InvalidQuantityException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(GeneralPersistenceException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		finally
		{
			if (em.getTransaction().isActive()) em.getTransaction().rollback();
		//	em.clear();
		}
			
	}

	

	public Order openOrder(int customerId)
			throws CustomerDoesNotExistException, OrderAlreadyOpenException, GeneralPersistenceException {
		
		
		try {
			em.getTransaction().begin();
			Order newOrder = super.openOrder(customerId);
			em.getTransaction().commit();
			return newOrder;
		} catch(CustomerDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(OrderAlreadyOpenException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(GeneralPersistenceException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		finally
		{
			if (em.getTransaction().isActive()) em.getTransaction().rollback();
		//	em.clear();
		}
	}

	public void submit(int customerId) throws CustomerDoesNotExistException,
			OrderNotOpenException, NoLineItemsException, GeneralPersistenceException {
		try
		{
			em.getTransaction().begin();
			super.submit(customerId);
			em.getTransaction().commit();
		}
		catch(CustomerDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(NoLineItemsException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(GeneralPersistenceException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		finally
		{
			if (em.getTransaction().isActive()) em.getTransaction().rollback();
		//	em.clear();
		}
	}

	public AbstractCustomer loadCustomer(int customerId) throws CustomerDoesNotExistException, GeneralPersistenceException {
		boolean startedHere = false;
		try
		{
		
		if(!em.getTransaction().isActive()) 
		{
			em.getTransaction().begin();
			startedHere = true;
		}
		AbstractCustomer customer = super.loadCustomer(customerId);
		em.refresh(customer);
		if(startedHere)
		{
			em.getTransaction().commit();
		}
		return customer;
		}
		finally
		{
			if (em.getTransaction().isActive() && startedHere) em.getTransaction().rollback();
		//	em.clear();
		}
	}

	@Override
	public void removeLineItem(int customerId, int productId) throws CustomerDoesNotExistException, OrderNotOpenException, ProductDoesNotExistException, NoLineItemsException, GeneralPersistenceException 
	{
		
		try
		{
			em.getTransaction().begin();
			super.removeLineItem(customerId, productId);
			em.getTransaction().commit();
		}
		catch(CustomerDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(OrderNotOpenException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(ProductDoesNotExistException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(GeneralPersistenceException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		catch(NoLineItemsException e)
		{
			em.getTransaction().rollback();
			throw e;
		}
		finally
		{
			if (em.getTransaction().isActive()) em.getTransaction().rollback();
		//	em.clear();
		}
	}

}
