import junit.framework.*;
import zero.tools.test.*;
import zero.xml.*;

String htmlForAtomFeed(myTest,testsSuite)
{
	StringWriter writer = new StringWriter();
	def html = new groovy.xml.MarkupBuilder(writer);
	html.html {
		h2 
		{
			a(href:"/resources/ZeroTestRunner/${myTest}?format=atom")
			{
				"Result for Suite:${myTest}";
			}
		}
		h3 "Test Summary";
		table {
			tr {
				th (align:'left') {div "Total Suite Time (ms)"};
				td "${testsSuite.suiteTime}";
			}
			tr {
				th (align:'left') {div "Number of Tests"};
				td "${testsSuite.testCaseCount}"
			}
			tr {
				th (align:'left') {div "Number of Tests that Passed"};
				td "${testsSuite.testCasePassed}"
			}
			tr {
				th (align:'left') {div "Number of Tests that Failed"};
				td "${testsSuite.failureCount}"
			}
			tr {
				th (align:'left') {div "Number of Tests that had errors"};
				td "${testsSuite.errorCount}"
			}
		}
		h3 "Test Details";
		table {
			thead
			{
				tr
				{
					th (align:'left') {div "Test Name"};
					th (align:'left') {div "Test Result"};
					th (align:'left') {div "Test Time (ms)"};
					th (align:'left') {div "Test Failure"};
					th (align:'left') {div "Test Error"};
				}
			}
			tbody
			{
				for(testCase in testsSuite.tests)
				{
					for(theTest in testCase.value)
					{
						tr
						{
							td (valign: 'top')
							{
								div theTest.name;
							}
							td (valign: 'top')
							{
								if(theTest.pass)
									div "Test Passed";
								else
									div "Test Failed";
							}
						
							td (valign: 'top')
							{
								div theTest.testTime;
							}
						
							td (valign: 'top')
							{
								if(!theTest.pass)
								{
									if(theTest.failure)
										div theTest.failure;
								}
								else
									div "-";
							}
						
							td (valign: 'top')
							{
								if(!theTest.pass)
								{
									if(theTest.error)
									{
										font(size:'1') {
										div theTest.error;
										}
									}
								}
								else
									div "-";
							}
						}
					}
				}
			}
		}
	}
	return writer.toString();
}


def renderTest(format,tests,feed)
{
	if(format != null)
	{
		if(format[0].toLowerCase() == "json")
		{
			logger.FINE {"ZeroTestRunner: Format requested is JSON"};
			request.view="JSON";
			request.json.output=tests;
		}
		else if(format[0].toLowerCase() == "xml")
		{
			logger.FINE {"ZeroTestRunner: Format requested is XML"};
			request.view="XML";
			request.xml.output=tests;
	    	request.xml.rootElement='TestResults';
		}
		else if(format[0].toLowerCase() == "html")
		{
			logger.FINE {"ZeroTestRunner: Format requested is HTML"};
			request.view="htmlTest.gt";
			request.html.output=tests;
		}
		else if(format[0].toLowerCase() == "atom")
		{
			logger.FINE {"ZeroTestRunner: Format requested is ATOM"};
			request.view="atom";
			request.atom.output=feed;
		}
		else
		{
			logger.WARNING {"ZeroTestRunner: Invalid Format requested, returing XML"};
			request.view="XML";
			request.xml.output=tests;
		    request.xml.rootElement='TestResults';
		}
	}
	else
	{
		logger.FINE {"ZeroTestRunner: Format is Null, returning XML"};
		request.view="XML";
		request.xml.output=tests;
	    request.xml.rootElement='TestResults';
	}
	
    
	logger.FINE {"ZeroTestRunner: Rendering"};
	zero.core.views.ViewEngine.render();
}


def onList()
{
	logger.FINE {"ZeroTestRunner: onList()"};
	try
	{
		HashMap tests = new HashMap();
		ArrayList feed = new ArrayList();
		logger.FINE {"ZeroTestRunner: Creating result Listener"};
		
		logger.FINE {"ZeroTestRunner: Getting List of Tests from zero.config: ${app.tests}"};
		for (myTest in app.tests)
		{
			ZeroTestResultListener myListener = new ZeroTestResultListener();
			logger.FINE {"ZeroTestRunner: Creating Runner for -> ${myTest}"};
			zero.tools.test.ZeroTestRunner runner = new zero.tools.test.ZeroTestRunner(myTest, this.getClass().getClassLoader(),null);
			logger.FINE {"ZeroTestRunner: Setting Listener"};
			runner.setListener(myListener);
			logger.FINE {"ZeroTestRunner: Running Test Case"};
			def myTestCase = runner.doRun();
			if(myTestCase == null)
			{
				logger.WARNING {"ZeroTestRunner: Invalid Test Case: ${myTestCase}"};
				ZeroSuiteInfo resultInfo = new ZeroSuiteInfo();
				logger.FINE {"ZeroTestRunner: Adding to Map, getting Next"};
				tests.put(myTest,new HashMap().put(myTest,resultInfo));
				continue;
			}
			logger.FINE {"ZeroTestRunner: Test Ran, getting results"};
			testsSuite = myListener.getZeroSuiteInfo();
			logger.FINE {"ZeroTestRunner: Storing result"};
			if(request.params.format.get() != null)
			{
				if(request.params.format.get()[0].toLowerCase() == "atom")
				{
					HashMap entry = new HashMap();
					entry.put("id",myTest.hashCode());
					entry.put("title",myTest);
					entry.put("authorname","Build Master");
					entry.put("updated",new java.util.Date());
					entry.put("contenttype","HTML");
					entry.put("content",htmlForAtomFeed(myTest,testsSuite));
					feed.add(entry);
				}
			}
			
			tests.put(myTest,testsSuite);
		}
		logger.FINE {"ZeroTestRunner: Getting Requested Format"};
		def format = request.params.format;
		renderTest(format,tests,feed);
	}
	catch(Throwable e)
	{
		logger.SEVERE {"Error Executing Test Cases: ${e.getMessage()}"};
		request.error.exception = e;
		request.error.message = e.getMessage();
		request.status = 500
		request.view = "error"
		zero.core.views.ViewEngine.render()
	
	}
}



def onRetrieve()
{
	logger.FINE {"ZeroTestRunner: onRetrieve()"};
	try
	{
		HashMap tests = new HashMap();
		ArrayList feed = new ArrayList();
		logger.FINE {"ZeroTestRunner: Creating result Listener"};
		ZeroTestResultListener myListener = new ZeroTestResultListener();
		logger.FINE {"ZeroTestRunner: Setting Listener"};
		myTest = request.params.ZeroTestRunnerId[0]
		logger.FINE {"ZeroTestRunner: retrieve request for test: ${myTest}"};
		zero.tools.test.ZeroTestRunner runner = null
		def testMethod = request.params.test;
		if(testMethod.get() == null)
			runner = new zero.tools.test.ZeroTestRunner(myTest, this.getClass().getClassLoader(),null);
		else
			runner = new zero.tools.test.ZeroTestRunner(myTest, this.getClass().getClassLoader(),testMethod.get()[0]);
		logger.FINE {"ZeroTestRunner: Setting Listener"};
		runner.setListener(myListener);
		logger.FINE {"ZeroTestRunner: Calling doRun"};
		def myTestCase = runner.doRun();
		if(myTestCase == null)
		{
			logger.WARNING {"ZeroTestRunner: Invalid Test Case: ${myTestCase}"};
			ZeroSuiteInfo resultInfo = new ZeroSuiteInfo();
			logger.FINE {"ZeroTestRunner: Adding to Map, getting Next"};
			tests.put(myTest,new HashMap().put(myTest,resultInfo));
		}
		else
		{
			logger.FINE {"ZeroTestRunner: Test Ran, getting results"};
			testsSuite = myListener.getZeroSuiteInfo();
			if(request.params.format.get() != null)
			{
				if(request.params.format.get()[0].toLowerCase() == "atom")
				{
					HashMap entry = new HashMap();
					entry.put("id",myTest.hashCode());
					entry.put("title",myTest);
					entry.put("authorname","Build Master");
					entry.put("updated",new java.util.Date());
					entry.put("contenttype","HTML");
					entry.put("content",htmlForAtomFeed(myTest,testsSuite));
					feed.add(entry);
				}
			}
			tests.put(myTest,testsSuite);
		}
		logger.FINE {"ZeroTestRunner: Getting Requested Format"};
		def format = request.params.format;
		renderTest(format,tests,feed);
	}
	catch(Throwable e)
	{
		logger.SEVERE {"Error Executing Test Cases: ${e.getMessage()}"};
		request.error.exception = e;
		request.error.message = e.getMessage();
		request.status = 500
		request.view = "error"
		zero.core.views.ViewEngine.render()
	
	}

}