package zero.tools.test;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import zero.core.logging.LogHelper;

/**
 * 
 * @author Roland Barcia
 * 
 * Listener for JUnit 3.x style TestCases, Including GroovyTestCases
 * 
 */
public class ZeroTestResultListener implements junit.framework.TestListener {

	private ZeroSuiteInfo zeroSuiteInfo = new ZeroSuiteInfo();
	private Logger logger;
	

	public ZeroTestResultListener() {
		super();
		logger = Logger.getLogger(this.getClass().getName());
	}
	
	public void startSuite()
	{
		zeroSuiteInfo.startTime = System.currentTimeMillis();
	}
	
	
	public void endSuite()
	{
		zeroSuiteInfo.endTime = System.currentTimeMillis();
		zeroSuiteInfo.suiteTime = zeroSuiteInfo.endTime - zeroSuiteInfo.startTime;
	}

	public void addError(junit.framework.Test myTest, java.lang.Throwable e) {
		zeroSuiteInfo.errorCount++;
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"addError", "Entering Callback for test case errorTest Case "
						+ myTest.getClass().getName());
		ZeroResultInfo testObject = (ZeroResultInfo) zeroSuiteInfo.tests
				.get(((TestCase) myTest).getName());
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"addError",
				"Moving Exception Stack Trace to Result Object, Test Case "
						+ myTest.getClass().getName());
		StringWriter writer = new StringWriter();
		PrintWriter pWriter = new PrintWriter(writer);
		e.printStackTrace(pWriter);
		testObject.error =e.getLocalizedMessage();
	}

	public void addFailure(junit.framework.Test myTest,
			junit.framework.AssertionFailedError f) {
		zeroSuiteInfo.failureCount++;
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"addFailure",
				"Entering Callback for test case failure, Test Case "
						+ myTest.getClass().getName());
		ZeroResultInfo testObject = (ZeroResultInfo) zeroSuiteInfo.tests
				.get(((TestCase) myTest).getName());
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"addFailure",
				"Moving Exception StackTrace to Result Object, Test Case "
						+ myTest.getClass().getName());
		StringWriter writer = new StringWriter();
		PrintWriter pWriter = new PrintWriter(writer);
		f.printStackTrace(pWriter);
		testObject.failure = f.toString();
	}

	public void endTest(junit.framework.Test myTest) {
		
		zeroSuiteInfo.testCaseCount = zeroSuiteInfo.testCaseCount + myTest.countTestCases();
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "endTest",
				"Entering Callback for test case End, Test Case "
						+ myTest.getClass().getName());
		ZeroResultInfo testObject  = (ZeroResultInfo) zeroSuiteInfo.tests.get(((TestCase) myTest)
					.getName());
		testObject.endTime = System.currentTimeMillis();
		testObject.testTime = testObject.endTime - testObject.startTime;
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "endTest",
				"Checking for errors or failures, Test Case "
						+ myTest.getClass().getName());
		if (testObject.error == null && testObject.failure == null) {
			testObject.pass = true;
			LogHelper.log(logger, Level.FINE, this.getClass().getName(),
					"endTest", "Test Passed, Test Case "
							+ myTest.getClass().getName());
			zeroSuiteInfo.testCasePassed++;
		} else {
			testObject.pass = false;
			LogHelper.log(logger, Level.FINE, this.getClass().getName(),
					"endTest", "Test Failed, Test Case "
							+ myTest.getClass().getName());
		}
	}

	public void startTest(junit.framework.Test myTest) {
		ZeroResultInfo testObject = new ZeroResultInfo();
		testObject.startTime = System.currentTimeMillis();
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"startTest",
				"Entering Callback for test case Start, Test Case "
						+ myTest.getClass().getName());
		
		LogHelper.log(logger, Level.FINE, this.getClass().getName(),
				"startTest",
				"Initializing result object and holding in Hash, Test Case "
						+ myTest.getClass().getName());

		
		
		if (myTest instanceof TestCase) {
			
			LogHelper.log(logger, Level.FINE, this.getClass().getName(),
					"startTest",
					"Initializing result object and holding in Hash");
			testObject.name = ((TestCase) myTest).getName();
			testObject.testClass = myTest.getClass().getName();
			zeroSuiteInfo.tests.put(testObject.name, testObject);
		}
		
		
		

	}

	public ZeroSuiteInfo getZeroSuiteInfo() {
		return zeroSuiteInfo;
	}

	

	
}