package zero.tools.test;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.textui.TestRunner;
import zero.core.logging.LogHelper;


public class ZeroTestRunner extends TestRunner 
{

	private ZeroTestResultListener listener;
	private String suiteName;
	private ClassLoader loader;
	private Logger logger;
	private String testName;
	
	
		
	public ZeroTestRunner(String suiteName,ClassLoader loader,String testName)
	{
		this.suiteName = suiteName;
		this.loader = loader;
		logger = Logger.getLogger(this.getClass().getName());
		this.testName = testName;
	}
	
	public TestResult doRun()
	{
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "doRun", "Attempting to run the following suite " + suiteName);
		Test test = null;
		
		 test = getTest(suiteName);
			
		if(testName != null)
			if(test instanceof TestCase)
			{
					((TestCase)test).setName(testName);
			}
		
		if(test == null)
		{
			LogHelper.log(logger, Level.WARNING, this.getClass().getName(), "doRun", "The following Suite Does not Exist, returning null: " + suiteName);
			return null;
		}
		
		
		return doRun(test);
	}
	
	@Override
	public TestResult doRun(Test test) {
		
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "doRun(test)", "Attempting to run the following suite " + test);
		TestResult result = createTestResult();
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "doRun(test)", "Result Object created for  " + suiteName);
		result.addListener(listener);
		
		listener.startSuite();
		test.run(result);
		listener.endSuite();
		
		return result;
	}


	public ZeroTestResultListener getListener() {
		return listener;
	}


	public void setListener(ZeroTestResultListener listener) {
		this.listener = listener;
	}
	
	@Override
	protected Class<? extends TestCase> loadSuiteClass(String suiteClassName) throws ClassNotFoundException {
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "loadSuiteClass", "Loading Suite " + suiteClassName + " with Classloader " + loader);
		try
		{
			Class<?> t = java.lang.Class.forName(suiteClassName,true,loader);
			LogHelper.log(logger, Level.FINE, this.getClass().getName(), "loadSuiteClass", "Loading as subclass of TestCase.");
			Class<? extends TestCase> test =  t.asSubclass(TestCase.class);
			return test;
		}
		catch(Throwable t)
		{
			LogHelper.log(logger, Level.WARNING, this.getClass().getName(), "loadSuiteClass", "Could Not Load TestCase");
			throw new ClassNotFoundException("Could Not Load "+ suiteClassName,t);
		}
		
		
	}
	
	@Override
	protected void runFailed(String message) {
		//Added to override default JUnit Behavior to Exit JVM.  We just want to move forward
		LogHelper.log(logger, Level.FINE, this.getClass().getName(), "runFailed", "runFailed -> " + message );
		
	}


	protected Test getTest(String suiteClassName, String testName) {
	    try {
	      Class clazz = loadSuiteClass(suiteClassName);
	      Constructor constructor = clazz.getConstructor(new Class[]{String.class});
	      Test test = (Test)constructor.newInstance(new Object[]{testName});

	     
	      return test;
	    } catch (ClassNotFoundException e) {
	      runFailed("Class not found \"" + suiteClassName + "\"");
	    } catch (InstantiationException e) {
	      runFailed("Could not create instance of class \"" + suiteClassName + "\" (" + e.getMessage() + ")");
	    } catch (IllegalAccessException e) {
	      runFailed("Could not create instance of class \"" + suiteClassName + "\" (" + e.getMessage() + ")");
	    } catch (InvocationTargetException e) {
	      runFailed("Could not create instance of class \"" + suiteClassName + "\" (" + e.getMessage() + ")");
	    } catch (NoSuchMethodException e) {
	      runFailed("Missing constructor \"" + suiteClassName + "\"(String) in class \"" + suiteClassName + "\"");
	    }
	    return null;
	  }

	
}
