package org.pwte.example.service.impl;

import org.pwte.example.service.*;
import org.pwte.example.domain.*;
import org.pwte.example.exception.*;
import zero.data.groovy.*;
import zero.data.handler.*;


public class CustomerOrderServiceImpl implements CustomerOrderServices{

	public AbstractCustomer loadCustomer(int customerId)
			throws CustomerDoesNotExistException {

		def db = Manager.create("pie");
		def typeRow = db.queryFirst(
				"SELECT type FROM CUSTOMER c WHERE c.CUSTOMER_ID = ${customerId}");
		String customerQuery = "SELECT c.CUSTOMER_ID,c.NAME,c.BUSINESS_VOLUME_DISCOUNT,c.BUSINESS_PARTNER,c.BUSINESS_DESCRIPTION,c.RESIDENTIAL_HOUSEHOLD_SIZE,c.RESIDENTIAL_FREQUENT_CUSTOMER,c.TYPE,o.ORDER_ID,o.STATUS,o.TOTAL,l.PRODUCT_ID,l.QUANTITY,l.AMOUNT,p.PRICE,p.DESCRIPTION FROM CUSTOMER c LEFT OUTER JOIN ORDERS o ON c.OPEN_ORDER = o.ORDER_ID LEFT OUTER JOIN (LINE_ITEM l JOIN PRODUCT p ON  l.PRODUCT_ID = p.PRODUCT_ID)ON o.ORDER_ID = l.ORDER_ID WHERE c.CUSTOMER_ID = ${customerId}";
		AbstractCustomer customer = null;
		if(typeRow == null)
		{
			throw new CustomerDoesNotExistException();
		}
		else if (typeRow.type.equals(CustomerType.RESIDENTAL.toString())) 
		{
			customer = db.query(customerQuery,new JoinResultHandler(ResidentialCustomer)).get(0);
		} 
		else if (typeRow.type.equals(CustomerType.BUSINESS.toString())) 
		{
			customer = db.query(customerQuery,new JoinResultHandler(BusinessCustomer)).get(0);
		} 
		else 
		{
			throw new CustomerDoesNotExistException();
		}
		
		if (customer == null)
			throw new CustomerDoesNotExistException();
		return customer;
	}

	public Order openOrder(int customerId)
			throws CustomerDoesNotExistException, OrderAlreadyOpenException {
		def db = Manager.create("pie");
		Order newOrder = new Order();
		db.inTransaction
		{
			AbstractCustomer customer = loadCustomer(customerId);
			if (customer.getOpenOrder() != null ) {
				throw new OrderAlreadyOpenException();
			}
			
			newOrder.setStatus(Order.Status.OPEN.toString());
			newOrder.setTotal(new BigDecimal(0));
			newOrder.setCustomer(customer);
			def orderId = db.insert("INSERT INTO ORDERS (STATUS,TOTAL,CUSTOMER_ID) VALUES (${Order.Status.OPEN.toString()},${new BigDecimal(0)},${customerId})",['ORDER_ID']);
			newOrder.setOrderId(orderId.intValue());
			db.update("UPDATE CUSTOMER SET OPEN_ORDER = ${newOrder.orderId}");
		}
		return newOrder;
	}

	public void submit(int customerId) throws CustomerDoesNotExistException,
			OrderNotOpenException, NoLineItemsException {
		def db = Manager.create("pie");
		db.inTransaction
		{
			AbstractCustomer customer = loadCustomer(customerId);
			Order openOrder = customer.getOpenOrder();
			if (openOrder == null) {
				throw new OrderNotOpenException();
			}
			if (openOrder.getLineitems() == null
				|| openOrder.getLineitems().size() <= 0) {
				throw new NoLineItemsException();
			}
			db.update("UPDATE ORDERS SET STATUS = ${Order.Status.SUBMITTED.toString()} where ORDER_ID = ${openOrder.orderId}");
			db.update("UPDATE CUSTOMER SET OPEN_ORDER = NULL");
		};
	}

	public LineItem addLineItem(int customerId, int productId, long quantity)
			throws CustomerDoesNotExistException, OrderNotOpenException,InvalidQuantityException, ProductDoesNotExistException {
		def db = Manager.create("pie");
		if(quantity <= 0 ) throw new InvalidQuantityException();
		LineItem li = null;
		db.inTransaction
		{
			Product product = db.queryFirst("select * from Product where product_id = ${productId} ", Product.class);
			if(product == null) throw new ProductDoesNotExistException();
			AbstractCustomer customer = loadCustomer(customerId);
			Order openOrder = customer.getOpenOrder();
			if (openOrder == null) {
				throw new OrderNotOpenException();
			}
			BigDecimal amount = product.getPrice().multiply(new BigDecimal(quantity));
			db.update("UPDATE ORDERS SET TOTAL = ${amount.add(openOrder.getTotal())}");
			LineItem existingLineItem = db.queryFirst("select * from LINE_ITEM where ORDER_ID = ${openOrder.orderId} and PRODUCT_ID = ${productId}",LineItem.class);
			if(existingLineItem == null)
			{
				LineItem lineItem = new LineItem();
				lineItem.setOrderId(openOrder.getOrderId());
				lineItem.setProductId(product.getProductId());
				lineItem.setAmount(amount);
				lineItem.setProduct(product);
				lineItem.setQuantity(quantity);
				db.update("INSERT INTO LINE_ITEM (ORDER_ID,PRODUCT_ID,amount,quantity) VALUES (${lineItem.orderId},${lineItem.productId},${lineItem.amount},${lineItem.quantity}) ");
				li =  lineItem;
			}
			else
			{
				existingLineItem.setQuantity(existingLineItem.getQuantity()+ quantity);
				existingLineItem.setAmount(existingLineItem.getAmount().add(amount));
				db.update("UPDATE LINE_ITEM SET quantity = ${existingLineItem.getQuantity()} ,amount = ${existingLineItem.getAmount()} ");
				li =  existingLineItem;
			}
		}
		return li;
	}
	
	public void removeLineItem(int customerId,int productId ) throws CustomerDoesNotExistException, OrderNotOpenException, ProductDoesNotExistException, NoLineItemsException, GeneralPersistenceException
	{
		def db = Manager.create("pie");
		db.inTransaction
		{
			Product product = db.queryFirst("select * from Product where product_id = ${productId} ", Product.class);
			if(product == null) throw new ProductDoesNotExistException();
			AbstractCustomer customer = loadCustomer(customerId);
			Order openOrder = customer.getOpenOrder();
			if (openOrder == null) 
			{
				throw new OrderNotOpenException();
			}
			db.update("DELETE FROM LINE_ITEM WHERE PRODUCT_ID = ${productId} AND ORDER_ID = ${customer.openOrder.orderId}");
		}
	}

}